<?php
include_once('interfaces/interface.DbAccessMethods.php');


final class DbAccess extends DbAccessBase implements DbAccessMethods
{
	private static $Instance;
	private $Mysql;
	private $Result;
	

	
	public static function Singleton() {
		if (!isset(self::$Instance)) {
			$ClassName = __CLASS__;
			self::$Instance = new $ClassName();
		}
		return self::$Instance;
	}	
	private function __clone() {}
	private function __construct() {
		$this->Connect();
		$this->SetFetchType(MYSQLI_ASSOC);
		$this->SetEncoding('utf8');
	}	
	public function __destruct() {
		$this->FreeResult();
	}
	

	
	
	public function Query($Query) {
		$this->FreeResult();
		$this->Query = $Query;
		
		if (CONF_DEBUG_ENABLED) {
			$Debug = DebugLogger::Singleton();
			$Debug->StartQuery($Query);
		}
		
		$this->Result = $this->Mysql->query($Query);
		
		if (CONF_DEBUG_ENABLED) {
			$Num = ($this->Result instanceof mysqli_result) ? $this->NumRows() : 0;
			$Aff = ($this->Mysql instanceof mysqli) ? $this->AffectedRows() : 0;
			$Inf = ($this->Mysql instanceof mysqli) ? $this->Info() : null;
			$Debug->EndQuery($Num, $Aff, $Inf);
			if ($this->Result === false) $Debug->AddMessage($this->Mysql->error);
		}
		
		if ($this->Result === false) {
			trigger_error($this->Mysql->error, E_USER_ERROR);
			exit;
		}
	}
	
	
	

	public function FetchRow() {
		return $this->Result->fetch_array($this->FetchType);
	}

	public function FetchRows($KeyField=null, $ValueField=null) {
		$Result = array();
		while ($Row = $this->Result->fetch_array($this->FetchType)) {
			$Value = $ValueField ? $Row[$ValueField] : $Row;
			if ($KeyField) {
				$Result[$Row[$KeyField]] = $Value;
			} else {
				$Result[] = $Value;
			}
		}
		return $Result;
	}

	public function FetchCell($Field=0) {
		$Row = $this->Result->fetch_array(MYSQLI_BOTH);
		if ($Row===null) return null;
		return $Row[$Field];
	}
	
	public function FetchColumn($Field=0) {
		$Result = array();
		while ($Row = $this->Result->fetch_array(MYSQLI_BOTH)) {
			$Result[] = $Row[$Field];
		}
		return $Result;
	}

	public function FetchObject() {
		return $this->Result->fetch_object();
	}

	public function FetchObjects() {
		$Result = array();
		while ($Result[] = $this->Result->fetch_object());
		return $Result;		
	}	
	
	
	
	
	public function NumRows() {
		return $this->Result->num_rows;
	}

	public function AffectedRows() {
		return $this->Mysql->affected_rows;
	}
	
	public function InsertId() {
		return $this->Mysql->insert_id;
	}

	public function Info() {
		return $this->Mysql->info;
	}

	public function Stat() {
		return $this->Mysql->stat();
	}
	
	public function FreeResult() {
		if ($this->Result instanceof mysqli_result) {
			@$this->Result->free();
		}
	}

	
	
	
	
	
	private function Connect() {
		$Host = CONF_DBHOST;
		if ($Host[0]==':') {
			// connect throuth socket
			$Sock = substr($Host, 1);
			$this->Mysql = new mysqli(null, CONF_DBUSER, CONF_DBPASS, CONF_DBNAME, null, $Sock);
		} else {
			$this->Mysql = new mysqli(CONF_DBHOST, CONF_DBUSER, CONF_DBPASS, CONF_DBNAME);
		}
		if (mysqli_connect_errno()) {
			trigger_error('Cannot connect to database: '.mysqli_connect_error(), E_USER_ERROR);
			exit;
		}		
	}
}

?>