<?php

class ProxyExporter
{
	private $db;
	private $ProxyList;
	
	private $id;
	private $Type;
	private $File;
	private $Mail;
	private $MailTitle;
	private $FtpHost;
	private $FtpPort;
	private $FtpUser;
	private $FtpPass;
	private $FtpFile;
	private $FtpPassive;
	private $Rate;
	private $Created;
	private $Exported;
	private $Error;
	private $Worked;
	private $Contime;
	private $Time;
	private $Speed;
	private $Post;
	private $Referer;
	private $Cookies;
	private $Anon;
	private $Countries;
	private $Fresh;
	private $Uptime;
	private $Delimiter;

	
	
	
	public function __construct($id) {
		$this->db = Db::Singleton();
		$this->id = $id;
		$this->Load();
	}
	
	
	
	
	/**
	* Execute query to database for select proxy with needed params
	*/
	public static function Filter($Worked=true, $Contime=0, $Time=0, $Speed=0, $Post=true, $Referer=true, $Cookies=true, $Anon=0, $Countries='', $Fresh=0, $Uptime=0) {
		if (!empty($Countries)) {
			$Countries = preg_split('#\W+#s', $Countries);
			$Countries = array_map('strtoupper', $Countries);
			$Countries = array_map('addslashes', $Countries);			
		}
		$Where = '1'
			. ($Worked ? ' AND worked=1' : '')
			. ($Contime ? ' AND contime <= '.doubleval($Contime) : '')
			. ($Time ? ' AND time <= '.doubleval($Time) : '')
			. ($Speed ? ' AND speed >= '.intval($Speed) : '')
			. ($Post ? ' AND post=1' : '')
			. ($Referer ? ' AND referer=1' : '')
			. ($Cookies ? ' AND cookies=1' : '')
			. ($Anon ? ' AND anon >= '.intval($Anon) : '')
			. (empty($Countries) ? '' : " AND code_country IN('".implode("','", $Countries)."')")
			. ($Fresh ? ' AND (UNIX_TIMESTAMP() - UNIX_TIMESTAMP(date_check)) < '.intval($Fresh*60) : '')
			. ($Uptime ? ' AND uptime >= '.intval($Uptime*60) : '');
		Db::Singleton()->Query('SELECT proxy FROM proxy WHERE '.$Where);
	}
	
	public static function MakeDelimiter($Key) {
		switch ($Key) {
			case 'rn':
				$Delimiter = "\r\n";
				break;
			case 'n':
				$Delimiter = "\n";
				break;
			case 'br':
				$Delimiter = '<br/>';
				break;
			case 'comma':
				$Delimiter = ',';
				break;
			case 'dotcom':
				$Delimiter = ';';
				break;
			default:
				$Delimiter = "\r\n";
		}
		return $Delimiter;		
	}	

	public function Export() {
		if (empty($this->Error))
		{			
			$this->MakeProxyList();
			if (empty($this->ProxyList)) {
				$this->Error = 'Proxies is not found by filter';
			} else {				
				switch ($this->Type) {
					case 'file':
						$this->SaveToFile();
						break;
					case 'mail':
						$this->SendToMail();
						break;
					case 'ftp':
						$this->UploadToFtp();
						break;
					default:
						$this->Error = 'Unknown receiver type';
				}
			}
			$this->Save();
		}
		return empty($this->Error) ? true : false;
	}
	
	public function GetId() {
		return $this->id;
	}
	
	public function GetError() {
		return $this->Error;
	}
	
	public function GetProxyList() {
		return $this->ProxyList;
	}

	public function GetDescription() {
		switch ($this->Type) {
			case 'file': return $this->File;
			case 'mail': return $this->Mail;
			case 'ftp': return $this->FtpHost.' '.$this->FtpFile;
		}
		return null;
	}
	
	
	
	
	
	
	
	private function Load() {
		$this->db->Query('SELECT * FROM export WHERE id='.intval($this->id));
		if ($this->db->NumRows()==0) {
			$this->Error = 'Record with ID='.$this->id.' is not found in database';
			return false;
		}
		$Row = $this->db->FetchRow();
		$this->Type = $Row['type'];
		$this->File = $Row['file'];
		$this->Mail = $Row['mail'];
		$this->MailTitle = $Row['mail_title'];
		$this->FtpHost = $Row['ftp_host'];
		$this->FtpPort = $Row['ftp_port'];
		$this->FtpUser = $Row['ftp_user'];
		$this->FtpPass = $Row['ftp_pass'];
		$this->FtpFile = $Row['ftp_file'];
		$this->FtpPassive = $Row['ftp_passive'];
		$this->Rate = $Row['rate'];
		$this->Created = $Row['created'];
		$this->Exported = $Row['exported'];
		$this->Error = null;
		$this->Worked = $Row['worked'];
		$this->Contime = $Row['contime'];
		$this->Time = $Row['time'];
		$this->Speed = $Row['speed'];
		$this->Post = $Row['post'];
		$this->Referer = $Row['referer'];
		$this->Cookies = $Row['cookies'];
		$this->Anon = $Row['anon'];
		$this->Countries = $Row['countries'];
		$this->Fresh = $Row['fresh'];
		$this->Uptime = $Row['uptime'];
		$this->Delimiter = $Row['delimiter'];
		return true;
	}

	private function Save() {
		$this->db->Query('UPDATE export SET'
			. " exported = NOW(),"
			. " error = '".addslashes($this->Error)."'"
			. " WHERE id=".intval($this->id));
	}
	
	private function MakeProxyList() {
		$this->ProxyList = '';
		self::Filter(
			$this->Worked,
			$this->Contime,
			$this->Time,
			$this->Speed,
			$this->Post,
			$this->Referer,
			$this->Cookies,
			$this->Anon,
			$this->Countries,
			$this->Fresh,
			$this->Uptime);
		$Delimiter = self::MakeDelimiter($this->Delimiter);
		while ($Row = $this->db->FetchRow()) {
			$this->ProxyList .= $Row['proxy'] . $Delimiter;
		}
	}
	
	private function SaveToFile() {		
		if (file_put_contents($this->File, $this->ProxyList) === false) {
			$this->Error = 'Cannot write file. Check permissions';
		}
		chmod($this->File, 0777);		
	}
	
	private function SendToMail() {
		if (mail($this->Mail, $this->MailTitle, $this->ProxyList, CONF_MAIL_EXTRA_HEADERS) == false) {
			$this->Error = 'Cannot send mail';
		}
	}
	
	private function UploadToFtp() {
		$Ftp = new FtpAccess($this->FtpHost, $this->FtpPort, $this->FtpUser, $this->FtpPass, CONF_FTP_TIMEOUT);
		if ($Ftp->GetError()) {$this->Error = $Ftp->GetError();	return;}
		
		if ($this->FtpPassive) {
			$Ftp->Passive(true);
			if ($Ftp->GetError()) {$this->Error = $Ftp->GetError();	return;}
		}
		
		$TmpFile = 'tmp/ftp'.rand(1111,9999).'.txt';
		if (file_put_contents($TmpFile, $this->ProxyList) === false) {
			$this->Error = 'Cannot write temp file: '.$TmpFile.' - check permissions';
			return;
		}
		
		$Ftp->Upload($TmpFile, $this->FtpFile);
		unlink($TmpFile);
		
		if ($Ftp->GetError()) {$this->Error = $Ftp->GetError();	return;}
	}
}

?>