<?php

class Skin extends Smarty
{
	private $SkinName;
	private $ModuleName;
	private $Languages;
	private $LangName;
	private $LangCode;
	private $Messages;
	private $Errors;
	private $FormErrors;
	

	
	
	
	public function __construct() {
		parent::__construct();
		$this->Languages = self::LoadLanguagesList();
		$this->Messages =& $_SESSION['Skin']['Messages'];
		$this->Errors =& $_SESSION['Skin']['Errors'];
		$this->FormErrors =& $_SESSION['Skin']['FormErrors'];
						
		$this->SetSkinName(CONF_DEFAULT_SKIN);
		$this->SetLangByName(CONF_DEFAULT_LANGUAGE);		
	
		$this->assign_by_ref('Skin', $this);
		$this->assign_by_ref('Dispatcher', Dispatcher::Singleton());

		$this->compile_dir = 'templates_c/';
		
		$this->register_modifier('link', array(Dispatcher::Singleton(), 'PathToAbsoluteSynonym'));
		$this->register_outputfilter(array($this, 'AllUrlsToAbsoluteSynonyms'));
		if (CONF_STRIP_WHITESPACES) $this->load_filter('output','trimwhitespace');
	}


	
	
	
	public function LoadLanguage($ModuleName) {
		$this->config_load('_common.lng');
		$this->config_load($ModuleName.'.lng');
	}

	public function SetSkinName($SkinName) {
		$this->SkinName = $SkinName;
		$this->template_dir = 'skins/'.$this->SkinName.'/templates/';
	}

	public function SetModuleName($ModuleName) {
		$this->ModuleName = $ModuleName;
		$this->LoadLanguage($ModuleName);
	}
	
	public function SetLangByName($Name) {
		if ($Code = $this->GetLangCodeByName($Name)) {
			$this->LangName = $Name;
			$this->LangCode = $Code;
			$this->config_dir = 'languages/'.$this->LangName.'/';
		}
	}

	public function SetLangByCode($Code) {
		if ($Name = $this->GetLangNameByCode($Code)) {
			$this->LangName = $Name;
			$this->LangCode = $Code;
			$this->config_dir = 'languages/'.$this->LangName.'/';
		}
	}
	
	
	
	
	
	
	
	public function GetModuleName() {
		return $this->ModuleName;
	}
	
	public function GetSkinName() {
		return $this->SkinName;
	}
	
	public function GetLanguages() {
		return $this->Languages;
	}
	
	public function GetLangName() {
		return $this->LangName;
	}
	
	public function GetLangCode() {
		return $this->LangCode;
	}

	public function GetLangNameByCode($Code) {
		if (isset($this->Languages[$Code])) return $this->Languages[$Code];
		return false;
	}

	public function GetLangCodeByName($Name) {
		return array_search($Name, $this->Languages);
	}
	
	public function Lang($Name=null) {
		return $this->get_config_vars($Name);
	}
	
	public function GetErrors() {
		return $this->Errors;
	}
	
	public function GetMessages() {
		return $this->Messages;
	}
	
	public function GetFormError($FieldName) {
		if (isset($this->FormErrors[$FieldName])) return $this->FormErrors[$FieldName];
		return null;
	}
	
	public function GetSkinDir() {
		return 'skins/'.$this->SkinName.'/';
	}
	
	public function GetCssDir() {
		return $this->GetSkinDir().'css/';
	}

	public function GetImagesDir() {
		return $this->GetSkinDir().'images/';
	}

	public function GetTemplatesDir() {
		return $this->GetSkinDir().'templates/';
	}
	
	public function GetRootUrl() {
		return CONF_URL_ROOT;
	}	
	
	public function GetCssUrl() {
		return CONF_URL_ROOT . $this->GetCssDir();
	}

	public function GetImagesUrl() {
		return CONF_URL_ROOT . $this->GetImagesDir();
	}

	
	
	
	
	public function AddError() {
		$this->Errors[] = $this->PrepareMessage(func_get_args());
	}
	
	public function AddMessage() {
		$this->Messages[] = $this->PrepareMessage(func_get_args());
	}
	
	public function AddFormError($FieldName) {
		$Args = func_get_args();
		unset($Args[0]);
		$this->FormErrors[$FieldName][] = $this->PrepareMessage($Args);
	}
	
	
	
	
	
	public function IsErrors() {
		return !empty($this->Errors);
	}
	
	public function IsMessages() {
		return !empty($this->Messages);
	}

	public function IsFormErrors() {
		return !empty($this->FormErrors);
	}

	public function IsFormError($FieldName) {
		return isset($this->FormErrors[$FieldName]);
	}
	
	public function IsTemplate($TemplateName) {
		return is_file($this->GetTemplatesDir().$this->ModuleName.'/'.$TemplateName);		
	}

	
	
	
	public function ClearMessages() {
		$this->Messages = array();
	}

	public function ClearErrors() {
		$this->Errors = array();
	}

	public function ClearFormError($FieldName) {
		if (isset($this->FormErrors[$FieldName])) {
			unset($this->FormErrors[$FieldName]);
		}
	}
	
	public function ClearFormErrors() {
		$this->FormErrors = array();
	}
	
	public function ClearAllMessages() {
		$this->ClearMessages();
		$this->ClearErrors();
		$this->ClearFormErrors();		
	}
		
	
	

	
	public function DisplayTemplate($TemplateName) {
		$this->display('Header.tpl');
		$this->display($this->ModuleName.'/'.$TemplateName);
		$this->display('Footer.tpl');
	}
	
	public function DisplayFormError($FieldName) {
		if ($this->IsFormError($FieldName)) {
			$this->assign('FieldName', $FieldName);
			$this->display('FormError.tpl');			
		}
	}

	public function FetchMail($TemplateName) {
		$template_dir = $this->template_dir;
		$this->template_dir = 'languages/';
		$Text = $this->fetch($this->LangName.'/'.$TemplateName);
		$this->template_dir = $template_dir;
		return $Text;
	}

	public static function LoadLanguagesList() {
		require 'configs/Languages.php';
		return $Languages;
	}
	

	
	
	
	
	
	protected function AllUrlsToAbsoluteSynonyms($html) {
		$html = preg_replace_callback(
			'#(<\w+\s* [^<>]* (?:href|src|action) \s*=\s*)   (["][^"<>]*["]  |  [\'][^\'<>]*[\']  |  [^\s<>]*)   ()#six',
			array($this, 'AllUrlsToAbsoluteSynonymsCallback'),
			$html
		);		
		return $html;
	}
	
	protected function AllUrlsToAbsoluteSynonymsCallback($m) {
		$before = $m[1];
		$url = $m[2];
		$after = $m[3];
		$url = trim($url, '"\'');
		$pUrl = parse_url($url);
		if (isset($pUrl['scheme'])) return $before.'"'.$url.'"'.$after;
		return $before.'"'.Dispatcher::Singleton()->PathToAbsoluteSynonym($url).'"'.$after;
	}
	

	
	
	
	
	private function PrepareMessage($Message) {
		if (is_string($Message)) {
			$Config = $this->get_config_vars($Message);
			if ($Config !== null) return $Config;
		}		
		if (is_array($Message)) {
			$Result = '';
			foreach ($Message as $v) {
				$Result .= $this->PrepareMessage($v);
			}
			return $Result;
		}		
		return $Message;
	}	
}

?>