<?php

class MultiRequester
{
	private $mHandle;
	private $mStatus;
	private $Objects;
	
	private static $Codes = array(
		CURLM_CALL_MULTI_PERFORM => 'Multi requests perform',
		CURLM_OK => 'OK',
		CURLM_BAD_HANDLE => 'Bad handle',
		CURLM_BAD_EASY_HANDLE => 'Bad easy handle',
		CURLM_OUT_OF_MEMORY => 'Out of memory',
		CURLM_INTERNAL_ERROR => 'Internal error',
		CURLMSG_DONE => 'SG done',
		);
	
		
		
		
	
	public function __construct($Objects=null) {
		$this->mHandle = curl_multi_init();
		$this->Objects = array();
		if ($Objects != null) $this->AddRequesters($Objects);
	}
	
	public function __destruct() {
		curl_multi_close($this->mHandle);
	}

	public function __clone() {
		$this->mHandle = curl_multi_init();
		foreach ($this->Objects as $k=>$v) $this->Objects[$k] = clone $v;
	}
	
	public function __call($Method, $Params) {
		$Res = array();
		foreach ($this->Objects as $Obj) {
			$Res[] = call_user_func_array(array($Obj, $Method), $Params);
		}
		return $Res;
	}
	
	
	
	
	
	/**
	* @param array $Objects - entities of Requester class
	*/	
	public function AddRequesters($Objects) {
		foreach ($Objects as $v) $this->AddRequester($v);
	}
	
	public function AddRequester(Requester $Object) {
		$this->Objects[] = $Object;
	}
	
	public function GetRequesters() {
		return $this->Objects;
	}
	
	public function Count() {
		return count($this->Objects);
	}
	
	
	
	
	/**
	* Execute all Requesters in parallel mode
	* @return bool
	*/
	public function Exec() {
		foreach ($this->Objects as $Obj) {
			$Obj->RefreshEngine();
			curl_setopt_array($Obj->GetHandle(), $Obj->GetOptArray());
			curl_multi_add_handle($this->mHandle, $Obj->GetHandle());
		}

		do {
			$this->mStatus = curl_multi_exec($this->mHandle, $Running);
			usleep(1);
		} while ($Running > 0);

		foreach ($this->Objects as $Obj) {
			$Obj->SetResult( curl_multi_getcontent($Obj->GetHandle()) );
			curl_multi_remove_handle($this->mHandle, $Obj->GetHandle());
		}
		
		return $this->mStatus == CURLM_OK;
	}
	
	
	
	
	
	
	public function GetStatusCode() {
		return $this->mStatus;
	}
	
	public function GetStatusString() {
		if (isset(self::$Codes[$this->mStatus])) return self::$Codes[$this->mStatus];
		return 'Unknown status';
	}
}

?>