<?php

class Requester 
{
	private $Handle;
	private $Options;
	private $Result;
	private static $HttpCodes = array(
		100 => 'Continue',
		101 => 'Switching protocols',
		200 => 'OK',
		201 => 'Created',
		202 => 'Accepted',
		203 => 'Non-authoritative information',
		204 => 'No content',
		205 => 'Reset content',
		206 => 'Partial content',
		300 => 'Multiple choices',
		301 => 'Moved permanently',
		302 => 'Found',
		303 => 'See other',
		304 => 'Not modified',
		305 => 'Use proxy',
		306 => '(Unused)',
		307 => 'Temporary redirect',
		400 => 'Bad request',
		401 => 'Unauthorized',
		402 => 'Payment required',
		403 => 'Forbidden',
		404 => 'Not found',
		405 => 'Method not allowed',
		406 => 'Not acceptable',
		407 => 'Proxy authentication required',
		408 => 'Request timeout',
		409 => 'Conflict',
		410 => 'Gone',
		411 => 'Length required',
		412 => 'Precondition failed',
		413 => 'Request entity too large',
		414 => 'Request-URI too long',
		415 => 'Unsupported media type',
		416 => 'Requested range not satisfiable',
		417 => 'Expectation failed',
		500 => 'Internal server error',
		501 => 'Not implemented',
		502 => 'Bad gateway',
		503 => 'Service unavailable',
		504 => 'Gateway timeout',
		505 => 'HTTP version not supported',
		);
	
	
		
		
	
	public function __construct($url = null) {
		$this->Handle = curl_init();
		$this->Options = array();
		$this->SetDefaultOptions();
		if ($url) $this->SetUrl($url);
	}
	
	public function __destruct() {
		curl_close($this->Handle);
	}
	
	public function __clone() {
		$this->Handle = curl_init();
	}
	
	
	
	
	
	
	public function SetOpt($Name, $Value) {
		$this->Options[$Name] = $Value;
	}
	
	public function SetOptArray($Array) {
		foreach ($Array as $k=>$v) {
			$this->Options[$k] = $v;
		}
	}
	
	public function SetDefaultOptions() {
		$this->SetOptArray(array(
			CURLOPT_TIMEOUT => 30,
			CURLOPT_CONNECTTIMEOUT => 10,
			CURLOPT_USERAGENT => 'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)',
			//CURLOPT_FOLLOWLOCATION => true,
			CURLOPT_AUTOREFERER => true,
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_BINARYTRANSFER => true,
			));
	}

	public function SetUrl($url) {
		$this->SetOpt(CURLOPT_URL, $url);
	}
	
	public function SetPost($PostFields) {
		$this->SetOpt(CURLOPT_POST, true);
		$this->SetOpt(CURLOPT_POSTFIELDS, $PostFields);
	}
	
	public function SetCookieFile($FileName) {
		$this->SetOpt(CURLOPT_COOKIEJAR, $FileName);
		$this->SetOpt(CURLOPT_COOKIEFILE, $FileName);
	}

	// service function used by MultiRequester
	public function SetResult($Content) {
		$this->Result = $Content;
	}
	
	public function DisableSslVerify() {
		$this->SetOpt(CURLOPT_SSL_VERIFYPEER, false);
		$this->SetOpt(CURLOPT_SSL_VERIFYHOST, false);
	}

	public function RemoveOpt($Name) {
		if (isset($this->Options[$Name])) unset($this->Options[$Name]);
	}

	public function RemoveOptArray($Options) {
		foreach ($Options as $Name) {
			if (isset($this->Options[$Name])) unset($this->Options[$Name]);
		}
	}
	
	public function RemovePost() {
		$this->RemoveOpt(CURLOPT_POST);
		$this->RemoveOpt(CURLOPT_POSTFIELDS);
	}
	
	public function RemoveCookieFile() {
		$this->RemoveOpt(CURLOPT_COOKIEJAR);
		$this->RemoveOpt(CURLOPT_COOKIEFILE);
	}	
	
	public function ClearOptions() {
		$this->Options = array();
	}
	
	
	
	
	
	public function RefreshEngine() {
		curl_close($this->Handle);
		$this->Handle = curl_init();
	}
	
	public function Exec() {
		$this->RefreshEngine();
		curl_setopt_array($this->Handle, $this->Options);
		$this->Result = curl_exec($this->Handle);
		return $this->Result;
	}
	
	public function IsOk() {
		return (curl_errno($this->Handle) || curl_error($this->Handle)) ? false : true;
	}

	public function IsError() {
		return (curl_errno($this->Handle) || curl_error($this->Handle)) ? true : false;
	}
	
	
	
	
	
	public function GetUrl() {
		return $this->GetOpt(CURLOPT_URL);
	}
	
	public function GetResult() {
		return $this->Result;
	}
	
	public function GetHandle() {
		return $this->Handle;
	}
	
	public function GetVersion() {
		return curl_version();
	}
	
	public function GetOpt($Name) {
		if (isset($this->Options[$Name])) return $this->Options[$Name];
		return null;
	}
	
	public function GetOptArray() {
		return $this->Options;		
	}
	
	public function GetInfo($Opt=null) {
		if ($Opt) return curl_getinfo($this->Handle, $Opt);
		return curl_getinfo($this->Handle);		
	}
	
	public function GetHttpCode() {
		return $this->GetInfo(CURLINFO_HTTP_CODE);
	}
	
	public function GetHttpAnswer() {
		return self::HttpCode2string($this->GetHttpCode());
	}
	
	public function GetErrno() {
		return curl_errno($this->Handle);
	}

	public function GetError() {
		return curl_error($this->Handle);
	}	
	
	public static function HttpCode2string($Code) {
		if (isset(self::$HttpCodes[$Code])) return self::$HttpCodes[$Code];
		return null;
	}	
	
}

?>