<?php

final class Dispatcher
{
	private static $Instance;
	private $Synonymizer;
	private $Application;
	private $Skin;
	
	private $RootDir;
	private $LangCode;
	private $Path;
	private $ModuleName;
	private $ActionName;
	
	

	

	public static function Singleton() {
		if( !isset(self::$Instance) ) {
			$ClassName = __CLASS__;
			self::$Instance = new $ClassName();
		}
		return self::$Instance;
	}
	private function __clone() {}
	private function __construct() {
		$this->Synonymizer = new UrlSynonymizer();
	}	
	public function __destruct() {
		$_SESSION['dispatcher_previous_path'] = $this->GetPath();
	}

	
	

	
	

	public function Dispatch() {
		$this->Path = preg_replace('#^'.CONF_URL_BASE.'#', '', $_SERVER['REQUEST_URI']);
		
		// LANGUAGE
		if (CONF_LANGUAGES_ENABLED) {
			if (empty($this->Path) && CONF_REDIRECT_TO_LANGUAGE) $this->RedirectToLanguage();
			$this->LangCode = $this->DetachLangCode($this->Path);
			$Languages = Skin::LoadLanguagesList();
			if (!isset($Languages[$this->LangCode])) {
				trigger_error('Unknown language code', E_USER_ERROR);
				exit;
			}
			setcookie('LangCode', $this->LangCode, time()+CONF_COOKIES_CACHE_EXPIRE, '/');
		}
		
		// SYNONYM TO PATH
		$this->Path = $this->SynonymToPath($this->Path);
		$this->SetupEnvironment($this->Path);
		list($this->ModuleName, $this->ActionName) = self::ParsePath($this->Path);
		if (empty($this->ModuleName) || empty($this->ActionName)) {
			trigger_error('Cannot parse path', E_USER_ERROR);
			exit;
		}
	
		// DIRECTORIES AND FILES
		$ModuleDirectory = 'modules/'.$this->ModuleName.'/';
		$ModuleFile = $ModuleDirectory.'class.'.$this->ModuleName.'.php';
		$ActionFile = $ModuleDirectory.'action.'.$this->ActionName.'.php';
		clearstatcache();
		if (!is_file($ModuleFile) || !is_file($ActionFile)) {
			trigger_error('Cannot found requested module or action', E_USER_ERROR);
			exit;			
		}

		// SKIN
		$this->Skin = new Skin();
		$this->Skin->SetLangByCode($this->LangCode);
		$this->Skin->SetModuleName($this->ModuleName);
		
		// APPLICATION
		require_once($ModuleFile);
		$this->Application = new $this->ModuleName();
		$this->Application->SetSkin($this->Skin);
		$this->Application->ProcessAction($ActionFile);
	}
	
	
	
	
	
	
	public function SetRootDir($RootDir) {
		$this->RootDir = $RootDir;
	}

	public function ChangeLocation($Location = '') {
		$purl = parse_url($Location);
		if (!isset($purl['scheme'])) {
			$Location = $this->PathToAbsoluteSynonym($Location);
		}
		if (CONF_DEBUG_ENABLED) {
			$Debug = DebugLogger::Singleton();
			$Debug->AddMessage('Location: '.$Location);
		}
		header('Location: '.$Location);
		exit();
	}
	
	public function Refresh() {
		$this->ChangeLocation($this->GetPath());
	}
	
	public function Back() {
		if ($this->GetPreviousPath()) $this->ChangeLocation($this->GetPreviousPath());
	}

	
	
	
	public static function Response404() {
		header('HTTP/1.0 404 Not Found');
		die('404 Requested page is not found');
	}
	
	/**
	* @return array($ModuleName, $ActionName)
	*/
	public static function ParsePath($Path) {
		$purl = parse_url($Path);
		if (empty($purl['path'])) return false;
		$Path = preg_replace('#^'.CONF_URL_BASE.'#', '', $purl['path']);
		$Parts = explode('/', $Path);
		if (empty($Parts[0])
		||	empty($Parts[1])) {
			return false;
		}
		return array($Parts[0], $Parts[1]);
	}
	
	
	
	
	
	
	public function SynonymToPath($Synonym) {
		return $this->Synonymizer->SynonymToUrl($Synonym);
	}
	
	public function PathToSynonym($Path) {
		return $this->Synonymizer->UrlToSynonym($Path);
	}

	public function PathToAbsoluteSynonym($Path) {
		$Code = isset($this->LangCode) ? $this->LangCode .'/' : '';
		if (CONF_LANGUAGES_ENABLED) {
			return CONF_URL_ROOT . $Code . $this->PathToSynonym($Path);	
		} else {
			return CONF_URL_ROOT . $this->PathToSynonym($Path);
		}		
	}
	
	
	
	
	
	
	public function GetRootDir() {
		return $this->RootDir;
	}
	
	public function GetRootUrl() {
		return CONF_URL_ROOT;
	}
	
	public function GetPath() {
		return $this->Path;
	}
	
	public function GetPreviousPath() {
		return isset($_SESSION['dispatcher_previous_path']) ? $_SESSION['dispatcher_previous_path'] : null;
	}
	
	public function GetLangCode() {
		return $this->LangCode;
	}
	
	public function GetModuleName() {
		return $this->ModuleName;
	}
	
	public function GetActionName() {
		return $this->ActionName;
	}

	public function GetApplication() {
		return $this->Application;
	}
	
	public function GetSkin() {
		return $this->Skin;
	}

	public static function GetCmdParams() {
		if (isset($_SERVER['argv'])
		&&	isset($_SERVER['argc'])
		&&	intval(isset($_SERVER['argc'])) > 0
		&&	is_array($_SERVER['argv'])) {
			return $_SERVER['argv'];						
		}
		return array();
	}
	
	



	
	private function SetupEnvironment($Path) {
		$_SERVER['REQUEST_URI'] = CONF_URL_BASE . $Path;
		$purl = parse_url($Path);

		if (isset($purl['query'])) {
			parse_str($purl['query'], $_GET);
		} else {
			$_GET = array();
		}
		
		$_REQUEST = array_merge($_GET, $_REQUEST);
	}

	private function DetachLangCode(&$Path) {
		if (!preg_match('|^([^/]+)/(.*)$|', $Path, $m)) return false;
		$Path = $m[2];
		return $m[1];
	}
	
	private function RedirectToLanguage() {
		$Languages = Skin::LoadLanguagesList();
		if (isset($_COOKIE['LangCode'])) {
			header('Location: '.CONF_URL_ROOT.$_COOKIE['LangCode'].'/');
		} else {
			$LangCode = array_search(CONF_DEFAULT_LANGUAGE, $Languages);
			header('Location: '.CONF_URL_ROOT.$LangCode.'/');
		}
		exit;
	}
		
}

?>