<?php

class UrlSynonymizer
{
	private static $UrlSynonyms;
	
	
	
	public function __construct() {
		if( !isset(self::$UrlSynonyms) ) {
			require('configs/UrlSynonyms.php');
		}
	}
	
	
	
	
	public function SynonymToUrl($Request) {
		$Synonym = $this->SearchSynonymByAlias($Request);
		if (empty($Synonym)) return $Request;
		
		$purl = parse_url($Request);
		$Params = array();

		// ADD HIDDEN PARAMS
		if (isset($Synonym['hidden'])) {
			$Params = array_merge($Params, $Synonym['hidden']);
		}

		// ADD STATIC PARAMS
		if (isset($Synonym['static'])) {
			$Params = array_merge($Params, $Synonym['static']);
		}

		// ADD PARAMS ENCODED IN PATH
		if (isset($Synonym['params'])) {
			$ParamsPart = preg_replace('#'.preg_quote($Synonym['alias'],'#').'#', '', $purl['path']);
			foreach($Synonym['params'] as $k=>$v) {
				if (preg_match('#^'.$k.'$#', $ParamsPart, $ParsedParams)) {
					
					$ParamsPositions = $this->GetParamsPositions($v);
					$VarParams = $this->MakeParamsArray($ParsedParams, $ParamsPositions);
					$Params = array_merge($Params, $VarParams);

				}
			}
		}
		
		// ADD QUERY PARAMS
		if (isset($purl['query'])) {
			parse_str($purl['query'], $Query);
			$Params = array_merge($Params, $Query);
		}

		// MAKE URL
		$URL = $Synonym['path'];
		if (!empty($Params)) {
			$URL .= '?'. http_build_query($Params);
		}

		return $URL;
	}
	
	
	
	
	
	
	
	
	public function UrlToSynonym($Request) {
		$Synonym = $this->SearchSynonymByPath($Request);
		if (empty($Synonym)) return $Request;
		
		$purl = parse_url($Request);
		$ParamsPart = '';
		
		// ENCODE PARAMS INTO PATH
		if (isset($purl['query'])) {
			parse_str($purl['query'], $Params);
			
			// remove hidden parameters
			if (isset($Synonym['hidden'])) {
				$Params = array_diff_key($Params, $Synonym['hidden']);
			}
			
			if (isset($Synonym['params'])) {
				
				// make params part
				foreach ($Synonym['params'] as $Tags) {
					$VarParams = $this->GetParamsPositions($Tags);
					$ParamKeys = array_keys($Params);
					$Intersect = array_intersect($VarParams, $ParamKeys);
					if (count($Intersect) == count($VarParams)) {
						$ParamsPart = $Tags;
						foreach ($VarParams as $v) {
							$ParamsPart = preg_replace('#<'.$v.'>#', $Params[$v], $ParamsPart);
							unset($Params[$v]);
						}
						break;						
					}
				}
								
			}
		}

		// ADD QUERY PARAMS
		if (!empty($Params)) {
			$ParamsPart .= '?'.http_build_query($Params);
		}
		
		return $Synonym['alias'] . $ParamsPart;
	}	
	
	
	
	
	
	
	
	
	private function SearchSynonymByAlias($URL) {
		$purl = parse_url($URL);
		foreach(self::$UrlSynonyms as $Synonym) {
			if (empty($Synonym['alias'])) {
				if (empty($purl['path'])) return $Synonym;				
			}
			elseif (preg_match('#^'.preg_quote($Synonym['alias'],'#').'#', $purl['path'])) {
				return $Synonym;
			}
		}
		return null;
	}
	
	private function SearchSynonymByPath($URL) {
		$purl = parse_url($URL);
		foreach(self::$UrlSynonyms as $Synonym) {
			if ($Synonym['path'] == $purl['path']) {
				return $Synonym;
			}
		}
		return null;	
	}
	
	private function GetParamsPositions($ParamsTags) {
		$Params = array();
		if (preg_match_all('#<(.*?)>#', $ParamsTags, $m)) {
			$Params = array_merge(array(null), $m[1]);
			unset($Params[0]);
		}
		return $Params;		
	}
	
	private function MakeParamsArray($ParsedParams, $ParamsPositions) {
		$Params = array();
		foreach($ParamsPositions as $k=>$v) {
			if (isset($ParsedParams[$k])) {
				$Params[$v] = $ParsedParams[$k];	
			}			
		}
		return $Params;
	}
}

?>