<?php

final class Cron extends Application 
{
	protected $Process;
	protected $Config;
	protected $Log;
	

	
	
	public function __construct() {
		$this->Dispatcher = Dispatcher::Singleton();
		if (CONF_DEBUG_ENABLED) $this->Debug('debug.Cron.srz');
		$this->InitDb();
		$this->Process = ProcessController::Singleton();
		$this->Config = DbConfig::Singleton();
		$this->Log = new LogFile('scheduler.log');
	}
	
	public function Run() {
		if ($this->Config->GetParam('enabled') == 0) $this->Log->AddSystem('Scheduler disabled');
		else {
			$Params = Dispatcher::GetCmdParams();
			if (isset($Params[1])) {
				switch ($Params[1]) {
					case 'grabber':
						$this->Grabber();
						break;
					case 'checker':
						$this->Checker();
						break;
					case 'rechecker':
						$this->Rechecker();
						break;
					case 'export':
						$this->Export();
						break;					
				}
			} else {
				$this->Scheduler();
			}
		}
	}
	
	
	
	
	
	protected function IsAllUrlsGrabbed() {
		$this->db->Query('SELECT id FROM urls WHERE grabbed=0 LIMIT 0,1');
		return $this->db->NumRows() ? false : true;
	}
	
	protected function IsAllProxyChecked() {
		$this->db->Query('SELECT proxy FROM proxy WHERE checked=0 LIMIT 0,1');
		return $this->db->NumRows() ? false : true;
	}

	protected function ResetGrabber() {
		if ($this->IsAllProxyChecked()
		&&	$this->IsAllUrlsGrabbed()) {
			$this->Log->AddMessage('All proxies checked and all sources grabbed - need for grabber reset');
			$this->db->Query('UPDATE urls SET grabbed=0');			
			$this->Log->AddMessage('Grabber has been reset');
			return true;
		}
		return false;
	}
	
	
	
	

	
	
	private function Scheduler() {
		$this->Log->AddSystem('Scheduler started');
		include_once('functions/SmartExec.php');
		$this->ResetHungProxies();
		$this->StartGrabber();
		$this->StartChecker();
		$this->StartRechecker();
		$this->StartExport();
	}
	
	private function StartProcesses($Count, $Param) {
		for ($i=1; $i<=$Count; $i++) {
			$Cmd = CONF_PHP_PATH.' '.$this->Dispatcher->GetRootDir().'cron.php '.$Param.' >/dev/null & 2>/dev/null';
			$Res = SmartExec($Cmd);
			if ($Res===false) {
				$this->Log->AddError('Cannot start process: '.$Cmd);
			} else {
				$this->Log->AddOk('Process started: '.$Cmd);
				if (!empty($Res)) $this->Log->AddMessage($Res);
			}
		}		
	}
	
	private function GetNeededCount($TotalCount, $Action=null) {
		$Where = $Action ? "WHERE action='".addslashes($Action)."'" : '';
		$this->db->Query('SELECT COUNT(*) FROM processes '.$Where);
		$Count = $this->db->FetchCell();
		$NeededCount = $TotalCount - $Count;
		if ($NeededCount < 0) $NeededCount = 0;
		$this->Log->AddMessage('Current processes count: '.$Count.'. Need start '.$NeededCount.' processes');		
		return $NeededCount;
	}
	
	private function ResetHungProxies() {
		$this->db->Query('UPDATE proxy SET worked=0, checked=0 WHERE checked=1 AND worked=0');
		$this->Log->AddMessage('Reset hung proxies: '.$this->db->AffectedRows().' processed');
	}
	
	
	
	
	private function StartGrabber() {
		$this->Log->AddMessage('Start grabber processes');
		
		if ($this->ResetGrabber()) exit;

		if ($this->IsAllUrlsGrabbed()) {
			$this->Log->AddMessage('No need for grabbing - all sources is grabbed');
			return;			
		}
		
		$Count = $this->GetNeededCount($this->Config->GetParam('grab_proc'), 'grabber');
		$this->StartProcesses($Count, 'grabber');
	}
	
	
	
	
	private function StartChecker() {
		$this->Log->AddMessage('Start checker processes');
		
		if ($this->IsAllProxyChecked()) {
			$this->Log->AddMessage('No need for checking - all proxies is checked');
			return;
		}

		$Count = $this->GetNeededCount($this->Config->GetParam('check_proc'), 'checker');
		$this->StartProcesses($Count, 'checker');
	}
	
	
	
	
	private function StartRechecker() {
		$this->Log->AddMessage('Start rechecker process');

		$Seconds = $this->Config->GetParam('recheck_rate') * 60;
		$this->db->Query('SELECT proxy FROM proxy'
			. ' WHERE worked=1 '
			. ' AND (UNIX_TIMESTAMP() - UNIX_TIMESTAMP(date_check)) > '.intval($Seconds)
			. ' LIMIT 0,1');
		if ($this->db->NumRows()==0) {
			$this->Log->AddMessage('No need for rechecking at this moment');
			return;
		}

		$Count = $this->GetNeededCount($this->Config->GetParam('recheck_proc'), 'rechecker');
		$this->StartProcesses($Count, 'rechecker');
	}
	
	
	
	
	private function StartExport() {
		$this->Log->AddMessage('Start export processes');
		
		$this->db->Query('SELECT id FROM export'
			. ' WHERE (UNIX_TIMESTAMP() - UNIX_TIMESTAMP(exported)) > (rate * 60)'
			. ' LIMIT 0,1');
		if ($this->db->NumRows()==0) {
			$this->Log->AddMessage('No need for export - no ready receivers');
			return;
		}
		
		$Count = $this->GetNeededCount($this->Config->GetParam('export_proc'), 'export');
		$this->StartProcesses($Count, 'export');
	}
	
	
	
	
	
	
	
	
		
	
	private function Grabber() {
		set_time_limit(CONF_PROC_DURATION + CONF_ZOMBIE_PROC_TIME);
		$LogFile = 'grabber_'.$this->Process->GetPnum() . '.log';
		$this->Process->SetLogFile($LogFile);
		$this->Log = new LogFile($LogFile);
		$this->ProcessAction('modules/Cron/action.Grabber.php');
	}

	private function Checker() {
		set_time_limit(CONF_PROC_DURATION + CONF_ZOMBIE_PROC_TIME);
		$LogFile = 'checker_'.$this->Process->GetPnum() . '.log';
		$this->Process->SetLogFile($LogFile);
		$this->Log = new LogFile($LogFile);
		$this->ProcessAction('modules/Cron/action.Checker.php');		
	}
	
	private function Rechecker() {
		set_time_limit(CONF_PROC_DURATION + CONF_ZOMBIE_PROC_TIME);
		$LogFile = 'rechecker_'.$this->Process->GetPnum() . '.log';
		$this->Process->SetLogFile($LogFile);
		$this->Log = new LogFile($LogFile);
		$this->ProcessAction('modules/Cron/action.Rechecker.php');		
	}
	
	private function Export() {
		set_time_limit(CONF_PROC_DURATION + CONF_ZOMBIE_PROC_TIME);
		$LogFile = 'export_'.$this->Process->GetPnum() . '.log';
		$this->Process->SetLogFile($LogFile);
		$this->Log = new LogFile($LogFile);
		$this->ProcessAction('modules/Cron/action.Export.php');		
	}

}

?>